"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createAtom = exports.Atom = void 0;
// https://github.com/substack/deep-freeze/blob/master/index.js
const deepFreeze = (o) => {
    // Don't freeze in production for better performance
    if (process.env.NODE_ENV === 'production') {
        return o;
    }
    if (o && (typeof o === 'object' || typeof o === 'function')) {
        Object.freeze(o);
        Object.getOwnPropertyNames(o).forEach((prop) => {
            if (Object.prototype.hasOwnProperty.call(o, prop)
                && o[prop]
                && (typeof o === 'object' || (prop !== 'caller' && prop !== 'callee' && prop !== 'arguments'))
                && (typeof o[prop] === 'object' || typeof o[prop] === 'function')
                && !Object.isFrozen(o[prop])) {
                deepFreeze(o[prop]);
            }
        });
    }
    return o;
};
class Atom {
    _value;
    _subscriptions;
    constructor(initialValue) {
        this._value = deepFreeze(initialValue);
        this._subscriptions = [];
        this.subscribe = this.subscribe.bind(this);
    }
    // @ts-expect-error
    // ts(2380) The return type of a 'get' accessor must be assignable to its 'set' accessor type.
    // 'T' could be instantiated with an arbitrary type which could be unrelated to 'DeepReadonly<T>'.
    get value() {
        return this._value;
    }
    set value(value) {
        const nextValue = deepFreeze(value);
        this._value = nextValue;
        this._subscriptions.forEach((cb) => cb(nextValue));
    }
    subscribe(subscription) {
        this._subscriptions = [...this._subscriptions, subscription];
        return () => {
            this._subscriptions = this._subscriptions.filter((cb) => cb !== subscription);
        };
    }
}
exports.Atom = Atom;
const createAtom = (initialValue) => new Atom(initialValue);
exports.createAtom = createAtom;
//# sourceMappingURL=Atom.js.map