"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useAtom = void 0;
const react_1 = require("react");
/** React hook that returns the stateful value of an atom and a function to update it  */
const useAtom = (atom) => {
    const [state, setState] = (0, react_1.useState)(() => atom.value);
    (0, react_1.useEffect)(() => {
        if (state !== atom.value)
            setState(() => atom.value);
        const unsubscribe = atom.subscribe((value) => setState(() => value));
        return unsubscribe;
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [atom]);
    const setValue = (0, react_1.useCallback)((value) => {
        // eslint-disable-next-line no-param-reassign
        atom.value = typeof value === 'function' ? value(atom.value) : value;
    }, [atom]);
    return [state, setValue];
};
exports.useAtom = useAtom;
//# sourceMappingURL=useAtom.js.map